/********************************************************************************************/
/* joypad.c - joystick / pad library. functions to read the controller.                     */
/*..........................................................................................*/
/* This has been updated to handle multiple controllers, straight from the joystick values. */
/*..........................................................................................*/
/* You do NOT need to call joyRead() anymore; reading is taken care of by the vsync routine */
/*..........................................................................................*/
/* Please note that the joyXXXX() routines should not be called directly; they are intended */
/* as assembler sub-routines, and so do NOT clear the parameter stack, which WILL cause     */
/* problems with HuC. I suppose I -could- fix that, but it's too much of a pain right now.  */
/*..........................................................................................*/
/* routines are named in a standardized way: joy[Identifier][State]. thus, joyUpDown checks */
/* to see if the UP directional is down (pressed), and joyDownUp check to see if the DOWN   */
/* directional is up (released). It may seem wierd, but it's very easy to remember.         */
/*..........................................................................................*/

#asm
_JOY_LEFT		= $80
_JOY_DOWN       = $40
_JOY_RIGHT      = $20
_JOY_UP         = $10

_JOY_I          = $01
_JOY_II         = $02
_JOY_SEL        = $04
_JOY_RUN        = $08
#endasm

char  mask;
char  Alpha;
char  Beta;

/*..........................................................................................*/
/* joyPress() - assembler subroutine to check button press states.                          */
/*..........................................................................................*/

joyPress()
{
#asm

  sxy
  lda		_mask
  
  ;------------------------------------------------------------------------
  ; was button up before?

  and		joyold,y
  bne		.ret_false	  	   	; no, button was down
  
  ;------------------------------------------------------------------------
  ; is button down now ?

  lda		_mask
  and		joy,y
  bne		.changed		; yes, state changed
  
  ;------------------------------------------------------------------------
  ; state didn't change, so it wasn't pressed this frame.
  
.ret_false:  
  cla
  clx
  bra		.out
  
  ;------------------------------------------------------------------------
  ; state changed -> was pressed this frame
  
.changed
  clx
  lda		#1
  
.out  
  rts
  
#endasm
}
/*..........................................................................................*/
/* joyRelease() - assembler subroutine to check button press states.                        */
/*..........................................................................................*/

joyRelease()
{
#asm

  sxy
  lda		_mask

  ;------------------------------------------------------------------------
  ; was button up before?

  and		joyold,y
  cmp		_mask
  bne		.ret_false	  	   	; no, button was down
  
  ;------------------------------------------------------------------------
  ; is button down now ?

  lda		_mask
  and		joy,y
  cmp		_mask
  bne		.changed		; yes, state changed
  
  ;------------------------------------------------------------------------
  ; state didn't change, so it wasn't pressed this frame.
  
.ret_false:  
  cla
  clx
  bra		.out
  
  ;------------------------------------------------------------------------
  ; state changed -> was pressed this frame
  
.changed
  clx
  lda		#1
  
.out  
  rts
#endasm
}
/*..........................................................................................*/
/* joyDown() - assembler subroutine to check button press states.                           */
/*..........................................................................................*/

joyDown()
{
#asm
    sxy
    lda		_mask
	and		joy,y
	bne		.ret_true
	
	cla
	clx
	bra		.out
	
.ret_true
	cla
	ldx		#1
	
.out
	rts
#endasm
}
/*..........................................................................................*/
/* joyUp() - assembler subroutine to check button press states.                             */
/*..........................................................................................*/

joyUp()
{
#asm
    sxy
	lda		_mask
	and		joy,y
	beq		.ret_true
	
	cla
	clx
	bra		.out
	
.ret_true
	cla
	ldx		#1
	
.out
	rts
#endasm
}
/********************************************************************************************/
/*  DIRECTIONAL KEYS.                                                                       */
/*..........................................................................................*/
/*  Left directional.                                                                       */
/*..........................................................................................*/
/* joyLeftPressed() - return true if D-Pad left pressed this frame.                         */
/*..........................................................................................*/

joyLeftPressed( pad )
char            pad;
{
  #asm

  sta		_Alpha
  stx		_Beta
  
  lda		#_JOY_LEFT
  sta		_mask
  jsr		_joyPress			; this runs rest of subroutine
  #endasm
}
/*..........................................................................................*/
/* joyLeftReleased() - return true if D-Pad left released this frame.                       */
/*..........................................................................................*/

joyLeftReleased( pad )
char			 pad;
{
  #asm
  lda		#_JOY_LEFT
  sta		_mask
  jsr		_joyRelease
  #endasm
}

/*..........................................................................................*/
/* joyLeftUp() - return true if D-Pad left is up this frame.                                */
/*..........................................................................................*/

joyLeftUp( pad )
char       pad;
{
#asm
	lda		#_JOY_LEFT
	sta		_mask
    jsr		_joyUp	
#endasm
}
/*..........................................................................................*/
/* joyLeftDown() - return true if D-Pad left is down this frame.                            */
/*..........................................................................................*/

joyLeftDown( pad )
char         pad;
{
#asm
    lda		#_JOY_LEFT
	sta		_mask
	jsr		_joyDown
#endasm
}
/********************************************************************************************/
/* Right Directional.                                                                       */
/*..........................................................................................*/
/* joyRightPressed() - return true if D-Pad right pressed this frame.                       */
/*..........................................................................................*/

joyRightPressed( pad )
char             pad;
{
  #asm
  lda		#_JOY_RIGHT
  sta		_mask
  jsr		_joyPress
  
  #endasm
}
/*..........................................................................................*/
/* joyRightReleased() - return true if D-Pad right released this frame.                     */
/*..........................................................................................*/

joyRightReleased( pad )
char              pad;
{
  #asm

  lda		#_JOY_RIGHT
  sta		_mask
  jsr		_joyRelease
  
  #endasm
}
/*..........................................................................................*/
/* joyRightUp() - return true if D-Pad right is up this frame.                              */
/*..........................................................................................*/

joyRightUp( pad )
char        pad;
{
#asm
	lda		#_JOY_RIGHT
	sta		_mask
	jsr		_joyUp
#endasm
}
/*..........................................................................................*/
/* joyRightDown() - return true if D-Pad right is down this frame.                          */
/*..........................................................................................*/

joyRightDown( pad )
char          pad;
{
#asm
	lda		#_JOY_RIGHT
	sta		_mask
	jsr		_joyDown
#endasm
}

/********************************************************************************************/
/* Up Directional.                                                                          */
/*..........................................................................................*/
/* joyUpPressed() - return true if D-Pad up pressed this frame.                             */
/*..........................................................................................*/

joyUpPressed( pad )
char          pad;
{
  #asm
  lda		#_JOY_UP
  sta  		_mask
  jsr		_joyPress
  #endasm
}
/*..........................................................................................*/
/* joyUpReleased() - return true if D-Pad Up released this frame.                           */
/*..........................................................................................*/

joyUpReleased( pad )
char           pad;
{
  #asm
  lda		#_JOY_UP
  sta		_mask
  jsr		_joyRelease
  #endasm
}
/*..........................................................................................*/
/* joyUpUp() - return true if D-Pad up is up this frame.                                    */
/*..........................................................................................*/

joyUpUp( pad )
char     pad;
{
#asm
  lda		#_JOY_UP
  sta		_mask
  jsr		_joyUp
#endasm
}
/*..........................................................................................*/
/* joyUpDown() - return true if D-Pad Up is down this frame.                                */
/*..........................................................................................*/

joyUpDown( pad )
char       pad;
{
#asm
  lda		#_JOY_UP
  sta		_mask
  jsr		_joyDown
#endasm
}
/********************************************************************************************/
/* Down Directional                                                                         */
/*..........................................................................................*/
/* joyDownPressed() - return true if D-Pad down pressed this frame.                         */
/*..........................................................................................*/

joyDownPressed( pad )
char            pad;
{
  #asm
  lda		#_JOY_DOWN
  sta		_mask
  jsr		_joyPress
  #endasm
}
/*..........................................................................................*/
/* joyDownReleased() - return true if D-Pad down released this frame.                       */
/*..........................................................................................*/

joyDownReleased( pad )
char             pad;
{
  #asm
  lda		#_JOY_DOWN
  sta		_mask
  jsr		_joyRelease
  #endasm
}
/*..........................................................................................*/
/* joyDownUp() - return true if D-Pad down is up this frame.                                */
/*..........................................................................................*/

joyDownUp( pad )
char       pad;
{
  #asm
  lda		#_JOY_DOWN
  sta		_mask
  jsr		_joyUp
  #endasm
}
/*..........................................................................................*/
/* joyDownDown() - return true if D-Pad down is down this frame.                            */
/*..........................................................................................*/

joyDownDown( pad )
char         pad;
{
  #asm
  lda		#_JOY_DOWN
  sta		_mask
  jsr		_joyDown
  #endasm
}
/********************************************************************************************/
/*  CONTROL KEYS.                                                                           */
/*..........................................................................................*/
/*  [RUN].                                                                                  */
/*..........................................................................................*/
/* joyRunPressed() - return true if D-Pad RUN pressed this frame.                           */
/*..........................................................................................*/

joyRunPressed( pad )
char           pad;
{
  #asm
  lda		#_JOY_RUN
  sta		_mask
  jsr		_joyPress
  #endasm
}
/*..........................................................................................*/
/* joyRunReleased() - return true if D-Pad RUN released this frame.                         */
/*..........................................................................................*/

joyRunReleased( pad )
char            pad;
{
  #asm
  lda		#_JOY_RUN
  sta		_mask
  jsr		_joyRelease
  #endasm
}
/*..........................................................................................*/
/* joyRunUp() - return true if D-Pad RUN is up this frame.                                  */
/*..........................................................................................*/

joyRunUp( pad )
char      pad;
{
  #asm
  lda		#_JOY_RUN
  sta		_mask
  jsr		_joyUp
  #endasm
}
/*..........................................................................................*/
/* joyRunDown() - return true if D-Pad RUN is down this frame.                              */
/*..........................................................................................*/

joyRunDown( pad )
char        pad;
{
  #asm
  lda		#_JOY_RUN
  sta		_mask
  jsr		_joyDown
  #endasm
}
/*..........................................................................................*/
/*  [SELECT].                                                                               */
/*..........................................................................................*/
/* joySelectPressed() - return true if D-Pad SELECT pressed this frame.                     */
/*..........................................................................................*/

joySelectPressed( pad )
char              pad;
{
  #asm
  lda		#_JOY_SEL
  sta		_mask
  jsr		_joyPress
  #endasm
}
/*..........................................................................................*/
/* joySelectReleased() - return true if D-Pad SELECT released this frame.                   */
/*..........................................................................................*/

joySelectReleased( pad )
char               pad;
{
  #asm
  lda		#_JOY_SEL
  sta		_mask
  jsr		_joyRelease
  #endasm
}
/*..........................................................................................*/
/* joySelectUp() - return true if D-Pad SELECT is up this frame.                            */
/*..........................................................................................*/

joySelectUp( pad )
char         pad;
{
  #asm
  lda		#_JOY_SEL
  sta		_mask
  jsr		_joyUp
  #endasm
}
/*..........................................................................................*/
/* joySelectDown() - return true if D-Pad SELECT is down this frame.                        */
/*..........................................................................................*/

joySelectDown( pad )
char           pad;
{
  #asm
  lda		#_JOY_SEL
  sta		_mask
  jsr		_joyDown
  #endasm
}
/********************************************************************************************/
/*  ACTION KEYS.                                                                            */
/*..........................................................................................*/
/* joy_I_Pressed() - return true if D-Pad I pressed this frame.                             */
/*..........................................................................................*/

joy_I_Pressed( pad )
char           pad;
{
  #asm
  lda		#_JOY_I
  sta		_mask
  jsr		_joyPress
  #endasm
}
/*..........................................................................................*/
/* joy_I_Released() - return true if D-Pad I released this frame.                           */
/*..........................................................................................*/

joy_I_Released( pad )
char            pad;
{
  #asm
  lda		#_JOY_I
  sta		_mask
  jsr		_joyRelease
  #endasm
}
/*..........................................................................................*/
/* joy_I_Up() - return true if D-Pad I is up this frame.                                    */
/*..........................................................................................*/

joy_I_Up( pad )
char      pad;

{
  #asm
  lda		#_JOY_I
  sta		_mask
  jsr		_joyUp
  #endasm
}
/*..........................................................................................*/
/* joy_I_Down() - return true if D-Pad I is down this frame.                                */
/*..........................................................................................*/

joy_I_Down( pad )
char        pad;
{
  #asm
  lda		#_JOY_I
  sta		_mask
  jsr		_joyDown
  #endasm
}
/*..........................................................................................*/
/*  II                                                                                      */
/*..........................................................................................*/
/* joy_II_Pressed() - return true if D-Pad II pressed this frame.                           */
/*..........................................................................................*/

joy_II_Pressed( pad )
char            pad;
{
  #asm
  lda		#_JOY_II
  sta		_mask
  jsr		_joyPress
  #endasm
}
/*..........................................................................................*/
/* joy_II_Released() - return true if D-Pad II released this frame.                         */
/*..........................................................................................*/

joy_II_Released( pad )
char        pad;
{
  #asm
  lda		#_JOY_II
  sta		_mask
  jsr		_joyRelease
  #endasm
}
/*..........................................................................................*/
/* joy_II_Up() - return true if D-Pad II is up this frame.                                  */
/*..........................................................................................*/

joy_II_Up( pad )
char       pad;
{
  #asm
  lda		#_JOY_II
  sta		_mask
  jsr		_joyUp
  #endasm
}
/*..........................................................................................*/
/* joy_II_Down() - return true if D-Pad II is down this frame.                              */
/*..........................................................................................*/

joy_II_Down( pad )
char         pad;
{
  #asm
  lda		#_JOY_II
  sta		_mask
  jsr		_joyDown
  #endasm
}
